@@ -62,12 +62,12 @@ enum qmi_device_expected_data_format {
 };
 
 enum qmi_data_endpoint_type {
-    QMI_DATA_ENDPOINT_TYPE_UNKNOWN   = 0x00,
-    QMI_DATA_ENDPOINT_TYPE_HSIC      = 0x01,
-    QMI_DATA_ENDPOINT_TYPE_HSUSB     = 0x02,
-    QMI_DATA_ENDPOINT_TYPE_PCIE      = 0x03,
-    QMI_DATA_ENDPOINT_TYPE_EMBEDDED  = 0x04,
-    QMI_DATA_ENDPOINT_TYPE_BAM_DMUX  = 0x05,
+	QMI_DATA_ENDPOINT_TYPE_UNKNOWN   = 0x00,
+	QMI_DATA_ENDPOINT_TYPE_HSIC      = 0x01,
+	QMI_DATA_ENDPOINT_TYPE_HSUSB     = 0x02,
+	QMI_DATA_ENDPOINT_TYPE_PCIE      = 0x03,
+	QMI_DATA_ENDPOINT_TYPE_EMBEDDED  = 0x04,
+	QMI_DATA_ENDPOINT_TYPE_BAM_DMUX  = 0x05,
 };
 
 void qmi_free(void *ptr);
@@ -78,38 +78,31 @@ typedef void (*qmi_destroy_func_t)(void *user_data);
 struct qmi_device;
 
 typedef void (*qmi_debug_func_t)(const char *str, void *user_data);
-typedef void (*qmi_sync_func_t)(void *user_data);
 typedef void (*qmi_shutdown_func_t)(void *user_data);
 typedef void (*qmi_discover_func_t)(void *user_data);
 
-struct qmi_device *qmi_device_new(int fd);
-
-struct qmi_device *qmi_device_ref(struct qmi_device *device);
-void qmi_device_unref(struct qmi_device *device);
+void qmi_device_free(struct qmi_device *device);
 
 void qmi_device_set_debug(struct qmi_device *device,
 				qmi_debug_func_t func, void *user_data);
 
-void qmi_device_set_close_on_unref(struct qmi_device *device, bool do_close);
-
-bool qmi_device_discover(struct qmi_device *device, qmi_discover_func_t func,
+int qmi_device_discover(struct qmi_device *device, qmi_discover_func_t func,
 				void *user_data, qmi_destroy_func_t destroy);
-bool qmi_device_shutdown(struct qmi_device *device, qmi_shutdown_func_t func,
+int qmi_device_shutdown(struct qmi_device *device, qmi_shutdown_func_t func,
 				void *user_data, qmi_destroy_func_t destroy);
 
-bool qmi_device_has_service(struct qmi_device *device, uint8_t type);
-bool qmi_device_get_service_version(struct qmi_device *device, uint8_t type,
+bool qmi_device_has_service(struct qmi_device *device, uint16_t type);
+bool qmi_device_get_service_version(struct qmi_device *device, uint16_t type,
 					uint16_t *major, uint16_t *minor);
 
-bool qmi_device_sync(struct qmi_device *device,
-		     qmi_sync_func_t func, void *user_data);
-bool qmi_device_is_sync_supported(struct qmi_device *device);
-
 enum qmi_device_expected_data_format qmi_device_get_expected_data_format(
 						struct qmi_device *device);
 bool qmi_device_set_expected_data_format(struct qmi_device *device,
 			enum qmi_device_expected_data_format format);
 
+struct qmi_device *qmi_device_new_qmux(const char *device);
+struct qmi_device *qmi_device_new_qrtr(void);
+
 struct qmi_param;
 
 struct qmi_param *qmi_param_new(void);
@@ -158,10 +151,10 @@ typedef void (*qmi_result_func_t)(struct qmi_result *result, void *user_data);
 typedef void (*qmi_create_func_t)(struct qmi_service *service, void *user_data);
 
 bool qmi_service_create(struct qmi_device *device,
-				uint8_t type, qmi_create_func_t func,
+				uint16_t type, qmi_create_func_t func,
 				void *user_data, qmi_destroy_func_t destroy);
 bool qmi_service_create_shared(struct qmi_device *device,
-				uint8_t type, qmi_create_func_t func,
+				uint16_t type, qmi_create_func_t func,
 				void *user_data, qmi_destroy_func_t destroy);
 
 struct qmi_service *qmi_service_ref(struct qmi_service *service);
