@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -52,7 +52,6 @@
 #include <ofono/phonebook.h>
 #include <ofono/sim.h>
 #include <ofono/sms.h>
-#include <ofono/ssn.h>
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 #include <ofono/stk.h>
@@ -534,7 +533,6 @@ static void calypso_post_sim(struct ofono_modem *modem)
 				data->dlcs[NETREG_DLC]);
 	ofono_call_meter_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_barring_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
-	ofono_ssn_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_volume_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 
 	mw = ofono_message_waiting_create(modem);
@@ -543,7 +541,6 @@ static void calypso_post_sim(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver calypso_driver = {
-	.name		= "calypso",
 	.probe		= calypso_probe,
 	.remove		= calypso_remove,
 	.enable		= calypso_enable,
@@ -552,16 +549,4 @@ static struct ofono_modem_driver calypso_driver = {
 	.post_sim	= calypso_post_sim,
 };
 
-static int calypso_init(void)
-{
-	return ofono_modem_driver_register(&calypso_driver);
-}
-
-static void calypso_exit(void)
-{
-	ofono_modem_driver_unregister(&calypso_driver);
-}
-
-OFONO_PLUGIN_DEFINE(calypso, "TI Calypso modem driver", VERSION,
-			OFONO_PLUGIN_PRIORITY_DEFAULT,
-			calypso_init, calypso_exit)
+OFONO_MODEM_DRIVER_BUILTIN(calypso, &calypso_driver)
