@@ -1,7 +1,8 @@
 /*
+ *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *  Copyright (C) 2009  Collabora Ltd. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -44,15 +45,16 @@
 #include <ofono/phonebook.h>
 #include <ofono/sim.h>
 #include <ofono/sms.h>
-#include <ofono/ssn.h>
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 
 #include <drivers/atmodem/vendor.h>
 
-static void g1_debug(const char *str, void *data)
+static void g1_debug(const char *str, void *user_data)
 {
-	ofono_info("%s", str);
+	const char *prefix = user_data;
+
+	ofono_info("%s%s", prefix, str);
 }
 
 /* Detect hardware, and initialize if found */
@@ -111,8 +113,8 @@ static int g1_enable(struct ofono_modem *modem)
 	if (chat == NULL)
 		return -EIO;
 
-	if (getenv("OFONO_AT_DEBUG") != NULL)
-		g_at_chat_set_debug(chat, g1_debug, NULL);
+	if (getenv("OFONO_AT_DEBUG"))
+		g_at_chat_set_debug(chat, g1_debug, "");
 
 	ofono_modem_set_data(modem, chat);
 
@@ -181,7 +183,6 @@ static void g1_post_sim(struct ofono_modem *modem)
 	ofono_netreg_create(modem, 0, "atmodem", chat);
 	ofono_call_meter_create(modem, 0, "atmodem", chat);
 	ofono_call_barring_create(modem, 0, "atmodem", chat);
-	ofono_ssn_create(modem, 0, "atmodem", chat);
 	ofono_sms_create(modem, OFONO_VENDOR_QUALCOMM_MSM, "atmodem", chat);
 	ofono_phonebook_create(modem, 0, "atmodem", chat);
 
@@ -191,7 +192,6 @@ static void g1_post_sim(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver g1_driver = {
-	.name		= "g1",
 	.probe		= g1_probe,
 	.remove		= g1_remove,
 	.enable		= g1_enable,
@@ -200,15 +200,4 @@ static struct ofono_modem_driver g1_driver = {
 	.post_sim	= g1_post_sim,
 };
 
-static int g1_init(void)
-{
-	return ofono_modem_driver_register(&g1_driver);
-}
-
-static void g1_exit(void)
-{
-	ofono_modem_driver_unregister(&g1_driver);
-}
-
-OFONO_PLUGIN_DEFINE(g1, "HTC G1 modem driver", VERSION,
-			OFONO_PLUGIN_PRIORITY_DEFAULT, g1_init, g1_exit)
+OFONO_MODEM_DRIVER_BUILTIN(g1, &g1_driver)
