@@ -127,14 +127,14 @@ static char **network_operator_technologies(struct network_operator_data *opd)
 			ntechs += 1;
 	}
 
-	techs = g_new0(char *, ntechs + 1);
+	techs = l_new(char *, ntechs + 1);
 	ntechs = 0;
 
 	for (i = 0; i < sizeof(opd->techs) * 8; i++) {
 		if (!(opd->techs & (1 << i)))
 			continue;
 
-		techs[ntechs++] = g_strdup(registration_tech_to_string(i));
+		techs[ntechs++] = l_strdup(registration_tech_to_string(i));
 	}
 
 	return techs;
@@ -345,7 +345,7 @@ static void set_network_operator_techs(struct network_operator_data *opd,
 					OFONO_NETWORK_REGISTRATION_INTERFACE,
 					"Technologies", DBUS_TYPE_STRING,
 					&technologies);
-	g_strfreev(technologies);
+	l_strv_free(technologies);
 }
 
 static char *get_operator_display_name(struct ofono_netreg *netreg)
@@ -365,7 +365,7 @@ static char *get_operator_display_name(struct ofono_netreg *netreg)
 	 */
 
 	if (opd == NULL) {
-		g_strlcpy(name, "", len);
+		l_strlcpy(name, "", len);
 		return name;
 	}
 
@@ -386,7 +386,7 @@ static char *get_operator_display_name(struct ofono_netreg *netreg)
 	spn = ofono_sim_get_spn(netreg->sim);
 
 	if (spn == NULL || strlen(spn) == 0) {
-		g_strlcpy(name, plmn, len);
+		l_strlcpy(name, plmn, len);
 		return name;
 	}
 
@@ -553,7 +553,7 @@ static void append_operator_properties(struct network_operator_data *opd,
 						DBUS_TYPE_STRING,
 						&technologies);
 
-		g_strfreev(technologies);
+		l_strv_free(technologies);
 	}
 
 	if (opd->eons_info && opd->eons_info->info) {
@@ -1124,21 +1124,12 @@ void __ofono_netreg_set_base_station_name(struct ofono_netreg *netreg,
 	if (netreg->base_station == NULL && name == NULL)
 		return;
 
-	if (netreg->base_station)
-		g_free(netreg->base_station);
+	l_free(netreg->base_station);
+	netreg->base_station = l_strdup(name);
 
-	if (name == NULL) {
-		netreg->base_station = NULL;
-
-		/*
-		 * We just got unregistered, set name to NULL
-		 * but don't emit signal
-		 */
-		if (netreg->current_operator == NULL)
-			return;
-	} else {
-		netreg->base_station = g_strdup(name);
-	}
+	/* We just got unregistered, set name to NULL, but don't emit signal */
+	if (!name && !netreg->current_operator)
+		return;
 
 	ofono_dbus_signal_property_changed(conn, path,
 					OFONO_NETWORK_REGISTRATION_INTERFACE,
@@ -1790,16 +1781,14 @@ static void netreg_unregister(struct ofono_atom *atom)
 	g_slist_free(netreg->operator_list);
 	netreg->operator_list = NULL;
 
-	if (netreg->base_station) {
-		g_free(netreg->base_station);
-		netreg->base_station = NULL;
-	}
+	l_free(netreg->base_station);
+	netreg->base_station = NULL;
 
 	if (netreg->settings) {
 		storage_close(netreg->imsi, SETTINGS_STORE,
 				netreg->settings, TRUE);
 
-		g_free(netreg->imsi);
+		l_free(netreg->imsi);
 		netreg->imsi = NULL;
 		netreg->settings = NULL;
 	}
@@ -1864,7 +1853,7 @@ static void netreg_load_settings(struct ofono_netreg *netreg)
 	if (netreg->settings == NULL)
 		return;
 
-	netreg->imsi = g_strdup(imsi);
+	netreg->imsi = l_strdup(imsi);
 
 	strmode = g_key_file_get_string(netreg->settings, SETTINGS_GROUP,
 					"Mode", NULL);
