@@ -47,6 +47,7 @@
 #include "storage.h"
 #include "simfs.h"
 #include "stkutil.h"
+#include "missing.h"
 
 /*
  * A new session object will be created if a USim/ISim applications are
@@ -185,8 +186,6 @@ static const char *const passwd_name[] = {
 
 static void sim_own_numbers_update(struct ofono_sim *sim);
 
-static GSList *g_drivers = NULL;
-
 static GSList *cached_pins = NULL;
 
 static const char *sim_passwd_name(enum ofono_sim_password_type type)
@@ -265,13 +264,13 @@ static char **get_own_numbers(GSList *own_numbers)
 	if (own_numbers)
 		nelem = g_slist_length(own_numbers);
 
-	ret = g_new0(char *, nelem + 1);
+	ret = l_new(char *, nelem + 1);
 
 	nelem = 0;
 	for (l = own_numbers; l; l = l->next) {
 		num = l->data;
 
-		ret[nelem++] = g_strdup(phone_number_to_string(num));
+		ret[nelem++] = l_strdup(phone_number_to_string(num));
 	}
 
 	return ret;
@@ -290,7 +289,7 @@ static char **get_locked_pins(struct ofono_sim *sim)
 		nelem += 1;
 	}
 
-	ret = g_new0(char *, nelem + 1);
+	ret = l_new(char *, nelem + 1);
 
 	nelem = 0;
 
@@ -298,7 +297,7 @@ static char **get_locked_pins(struct ofono_sim *sim)
 		if (sim->locked_pins[i] == FALSE)
 			continue;
 
-		ret[nelem] = g_strdup(sim_passwd_name(i));
+		ret[nelem] = l_strdup(sim_passwd_name(i));
 		nelem += 1;
 	}
 
@@ -346,14 +345,14 @@ static char **get_service_numbers(GSList *service_numbers)
 
 	nelem = g_slist_length(service_numbers) * 2;
 
-	ret = g_new0(char *, nelem + 1);
+	ret = l_new(char *, nelem + 1);
 
 	nelem = 0;
 	for (l = service_numbers; l; l = l->next) {
 		num = l->data;
 
-		ret[nelem++] = g_strdup(num->id);
-		ret[nelem++] = g_strdup(phone_number_to_string(&num->ph));
+		ret[nelem++] = l_strdup(num->id);
+		ret[nelem++] = l_strdup(phone_number_to_string(&num->ph));
 	}
 
 	return ret;
@@ -445,15 +444,14 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 	}
 
 	own_numbers = get_own_numbers(sim->own_numbers);
-
 	ofono_dbus_dict_append_array(&dict, "SubscriberNumbers",
 					DBUS_TYPE_STRING, &own_numbers);
-	g_strfreev(own_numbers);
+	l_strv_free(own_numbers);
 
 	locked_pins = get_locked_pins(sim);
 	ofono_dbus_dict_append_array(&dict, "LockedPins",
 					DBUS_TYPE_STRING, &locked_pins);
-	g_strfreev(locked_pins);
+	l_strv_free(locked_pins);
 
 	if (sim->service_numbers && sim->sdn_ready) {
 		service_numbers = get_service_numbers(sim->service_numbers);
@@ -461,7 +459,7 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 		ofono_dbus_dict_append_dict(&dict, "ServiceNumbers",
 						DBUS_TYPE_STRING,
 						&service_numbers);
-		g_strfreev(service_numbers);
+		l_strv_free(service_numbers);
 	}
 
 	if (sim->language_prefs)
@@ -517,15 +515,15 @@ static void pin_cache_update(const char *iccid, const char *pin)
 	struct cached_pin *cpins;
 
 	if (pin_cached != NULL) {
-		g_free(pin_cached->pin);
-		pin_cached->pin = g_strdup(pin);
+		l_free(pin_cached->pin);
+		pin_cached->pin = l_strdup(pin);
 		return;
 	}
 
 	cpins = g_new0(struct cached_pin, 1);
 
-	cpins->id = g_strdup(iccid);
-	cpins->pin = g_strdup(pin);
+	cpins->id = l_strdup(iccid);
+	cpins->pin = l_strdup(pin);
 	cached_pins = g_slist_prepend(cached_pins, cpins);
 }
 
@@ -538,8 +536,8 @@ static void pin_cache_remove(const char *iccid)
 
 	cached_pins = g_slist_remove(cached_pins, pin_cached);
 
-	g_free(pin_cached->pin);
-	g_free(pin_cached->id);
+	l_free(pin_cached->pin);
+	l_free(pin_cached->id);
 	g_free(pin_cached);
 }
 
@@ -750,7 +748,7 @@ static DBusMessage *sim_set_property(DBusConnection *conn, DBusMessage *msg,
 				goto error;
 
 			own = g_new0(struct ofono_phone_number, 1);
-			string_to_phone_number(value, own);
+			__string_to_phone_number(value, own);
 
 			own_numbers = g_slist_prepend(own_numbers, own);
 
@@ -833,7 +831,7 @@ static void sim_locked_cb(struct ofono_sim *sim, gboolean locked)
 	if (g_strcmp0(typestr, "pin") == 0)
 		pin_cache_update(sim->iccid, pin);
 
-	g_strfreev(locked_pins);
+	l_strv_free(locked_pins);
 
 	sim_pin_retries_check(sim);
 }
@@ -1182,7 +1180,7 @@ static void sim_iidf_read_cb(int ok, int length, int record,
 		clut_len = data[3] * 3;
 
 	iidf_id = efimg[3] << 8 | efimg[4];
-	sim->iidf_image = g_memdup(data, length);
+	sim->iidf_image = g_memdup2(data, length);
 
 	/* The path it the same between 2G and 3G */
 	path_len = sim_ef_db_get_path_3g(SIM_EFIMG_FILEID, path);
@@ -1414,13 +1412,11 @@ check:
 		sim->own_numbers = sim->new_numbers;
 
 		own_numbers = get_own_numbers(sim->own_numbers);
-
 		ofono_dbus_signal_array_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
 						"SubscriberNumbers",
 						DBUS_TYPE_STRING, &own_numbers);
-
-		g_strfreev(own_numbers);
+		l_strv_free(own_numbers);
 	} else {
 		g_slist_free_full(sim->new_numbers, g_free);
 	}
@@ -1505,7 +1501,7 @@ check:
 						"ServiceNumbers",
 						DBUS_TYPE_STRING,
 						&service_numbers);
-		g_strfreev(service_numbers);
+		l_strv_free(service_numbers);
 	}
 }
 
@@ -1664,7 +1660,8 @@ static void impi_read_cb(int ok, int total_length, int record,
 		return;
 	}
 
-	sim->impi = g_strndup((const char *)data + 2, data[1]);
+	if (validate_utf8_tlv(data))
+		sim->impi = l_strndup((const char *)data + 2, data[1]);
 }
 
 static void discover_apps_cb(const struct ofono_error *error,
@@ -1716,7 +1713,7 @@ static void sim_imsi_obtained(struct ofono_sim *sim, const char *imsi)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(sim->atom);
 
-	sim->imsi = g_strdup(imsi);
+	sim->imsi = l_strdup(imsi);
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
@@ -1924,7 +1921,7 @@ static void sim_efsst_read_cb(int ok, int length, int record,
 		goto out;
 	}
 
-	sim->efsst = g_memdup(data, length);
+	sim->efsst = g_memdup2(data, length);
 	sim->efsst_length = length;
 
 	/*
@@ -1963,7 +1960,7 @@ static void sim_efest_read_cb(int ok, int length, int record,
 		goto out;
 	}
 
-	sim->efest = g_memdup(data, length);
+	sim->efest = g_memdup2(data, length);
 	sim->efest_length = length;
 
 	/*
@@ -2007,7 +2004,7 @@ static void sim_efust_read_cb(int ok, int length, int record,
 		goto out;
 	}
 
-	sim->efust = g_memdup(data, length);
+	sim->efust = g_memdup2(data, length);
 	sim->efust_length = length;
 
 	/*
@@ -2150,20 +2147,20 @@ static void sim_efli_read_cb(int ok, int length, int record,
 	if (!ok)
 		return;
 
-	sim->efli = g_memdup(data, length);
+	sim->efli = g_memdup2(data, length);
 	sim->efli_length = length;
 }
 
 /* Detect whether the file is in EFli format, as opposed to 51.011 EFlp */
-static gboolean sim_efli_format(const unsigned char *ef, int length)
+static bool sim_efli_format(const unsigned char *ef, int length)
 {
 	int i;
 
 	if (length & 1)
-		return FALSE;
+		return false;
 
 	for (i = 0; i < length; i += 2) {
-		if (ef[i] == 0xff && ef[i+1] == 0xff)
+		if (ef[i] == 0xff && ef[i + 1] == 0xff)
 			continue;
 
 		/*
@@ -2171,86 +2168,43 @@ static gboolean sim_efli_format(const unsigned char *ef, int length)
 		 * characters while CB DCS language codes are in ranges
 		 * (0 - 15) or (32 - 47), so the ranges don't overlap
 		 */
-		if (g_ascii_isalpha(ef[i]) == 0)
-			return FALSE;
+		if (l_ascii_isalpha(ef[i]) == 0)
+			return false;
 
-		if (g_ascii_isalpha(ef[i+1]) == 0)
-			return FALSE;
+		if (l_ascii_isalpha(ef[i + 1]) == 0)
+			return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
-static GSList *parse_language_list(const unsigned char *ef, int length)
+static char **concat_lang_prefs(char **a, char **b)
 {
-	int i;
-	GSList *ret = NULL;
-
-	for (i = 0; i < length; i += 2) {
-		if (ef[i] > 0x7f || ef[i+1] > 0x7f)
-			continue;
-
-		/*
-		 * ISO 639 codes contain only characters that are coded
-		 * identically in SMS 7 bit charset, ASCII or UTF8 so
-		 * no conversion.
-		 */
-		ret = g_slist_prepend(ret, g_ascii_strdown((char *)ef + i, 2));
-	}
-
-	if (ret)
-		ret = g_slist_reverse(ret);
-
-	return ret;
-}
-
-static GSList *parse_eflp(const unsigned char *eflp, int length)
-{
-	int i;
-	char code[3];
-	GSList *ret = NULL;
-
-	for (i = 0; i < length; i++) {
-		if (iso639_2_from_language(eflp[i], code) == FALSE)
-			continue;
-
-		ret = g_slist_prepend(ret, g_strdup(code));
-	}
-
-	if (ret)
-		ret = g_slist_reverse(ret);
-
-	return ret;
-}
-
-static char **concat_lang_prefs(GSList *a, GSList *b)
-{
-	GSList *l, *k;
 	char **ret;
-	int i = 0;
-	int total = g_slist_length(a) + g_slist_length(b);
+	int i;
+	int j;
+	int total = l_strv_length(a) + l_strv_length(b);
 
 	if (total == 0)
 		return NULL;
 
-	ret = g_new0(char *, total + 1);
+	ret = l_new(char *, total + 1);
 
-	for (l = a; l; l = l->next)
-		ret[i++] = g_strdup(l->data);
+	for (i = 0; a && a[i]; i++)
+		ret[i] = a[i];
 
-	for (l = b; l; l = l->next) {
-		gboolean duplicate = FALSE;
-
-		for (k = a; k; k = k->next)
-			if (!strcmp(k->data, l->data))
-				duplicate = TRUE;
-
-		if (duplicate)
+	for (j = 0; b && b[j]; j++) {
+		if (l_strv_contains(a, b[j])) {
+			l_free(b[j]);
 			continue;
+		}
 
-		ret[i++] = g_strdup(l->data);
+		ret[i++] = b[j];
 	}
 
+	l_free(a);
+	l_free(b);
+
 	return ret;
 }
 
@@ -2261,23 +2215,24 @@ static void sim_efpl_read_cb(int ok, int length, int record,
 	struct ofono_sim *sim = userdata;
 	const char *path = __ofono_atom_get_path(sim->atom);
 	DBusConnection *conn = ofono_dbus_get_connection();
-	gboolean efli_format = TRUE;
-	GSList *efli = NULL;
-	GSList *efpl = NULL;
+	bool efli_format = true;
+	char **efli = NULL;
+	char **efpl = NULL;
 
 	if (!ok || length < 2)
 		goto skip_efpl;
 
-	efpl = parse_language_list(data, length);
+	efpl = sim_parse_language_list(data, length);
 
 skip_efpl:
 	if (sim->efli && sim->efli_length > 0) {
 		efli_format = sim_efli_format(sim->efli, sim->efli_length);
 
 		if (efli_format)
-			efli = parse_language_list(sim->efli, sim->efli_length);
+			efli = sim_parse_language_list(sim->efli,
+							sim->efli_length);
 		else
-			efli = parse_eflp(sim->efli, sim->efli_length);
+			efli = sim_parse_eflp(sim->efli, sim->efli_length);
 	}
 
 	/*
@@ -2293,13 +2248,14 @@ skip_efpl:
 	 */
 	if (efli_format) {
 		if (sim->efli_length >= 2 && sim->efli[0] == 0xff &&
-				sim->efli[1] == 0xff)
-			sim->language_prefs = concat_lang_prefs(NULL, efpl);
-		else
-			sim->language_prefs = concat_lang_prefs(efli, efpl);
-	} else {
+				sim->efli[1] == 0xff) {
+			l_strv_free(efli);
+			efli = NULL;
+		}
+
+		sim->language_prefs = concat_lang_prefs(efli, efpl);
+	} else
 		sim->language_prefs = concat_lang_prefs(efpl, efli);
-	}
 
 	if (sim->efli) {
 		g_free(sim->efli);
@@ -2307,14 +2263,6 @@ skip_efpl:
 		sim->efli_length = 0;
 	}
 
-	if (efli) {
-		g_slist_free_full(efli, g_free);
-	}
-
-	if (efpl) {
-		g_slist_free_full(efpl, g_free);
-	}
-
 	if (sim->language_prefs != NULL)
 		ofono_dbus_signal_array_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
@@ -2343,7 +2291,7 @@ static void sim_iccid_read_cb(int ok, int length, int record,
 
 	extract_bcd_number(data, length, iccid);
 	iccid[20] = '\0';
-	sim->iccid = g_strdup(iccid);
+	sim->iccid = l_strdup(iccid);
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
@@ -2357,7 +2305,7 @@ static void sim_iccid_changed(int id, void *userdata)
 	struct ofono_sim *sim = userdata;
 
 	if (sim->iccid) {
-		g_free(sim->iccid);
+		l_free(sim->iccid);
 		sim->iccid = NULL;
 	}
 
@@ -2377,7 +2325,7 @@ static void sim_efli_efpl_changed(int id, void *userdata)
 		return;
 
 	if (sim->language_prefs) {
-		g_strfreev(sim->language_prefs);
+		l_strv_free(sim->language_prefs);
 		sim->language_prefs = NULL;
 	}
 
@@ -2642,7 +2590,7 @@ static void sim_inserted_update(struct ofono_sim *sim)
 static void sim_free_early_state(struct ofono_sim *sim)
 {
 	if (sim->iccid) {
-		g_free(sim->iccid);
+		l_free(sim->iccid);
 		sim->iccid = NULL;
 	}
 
@@ -2653,7 +2601,7 @@ static void sim_free_early_state(struct ofono_sim *sim)
 	}
 
 	if (sim->language_prefs) {
-		g_strfreev(sim->language_prefs);
+		l_strv_free(sim->language_prefs);
 		sim->language_prefs = NULL;
 	}
 
@@ -2706,7 +2654,7 @@ static void aid_session_free(gpointer data)
 static void sim_free_main_state(struct ofono_sim *sim)
 {
 	if (sim->imsi) {
-		g_free(sim->imsi);
+		l_free(sim->imsi);
 		sim->imsi = NULL;
 	}
 
@@ -2773,8 +2721,7 @@ static void sim_free_main_state(struct ofono_sim *sim)
 		sim->isim_context = NULL;
 	}
 
-	if (sim->impi)
-		g_free(sim->impi);
+	l_free(sim->impi);
 
 	if (sim->aid_list) {
 		g_slist_free_full(sim->aid_list,
@@ -2813,7 +2760,7 @@ static void sim_set_locked_pin(struct ofono_sim *sim,
 				OFONO_SIM_MANAGER_INTERFACE, "LockedPins",
 				DBUS_TYPE_STRING, &locked_pins);
 
-	g_strfreev(locked_pins);
+	l_strv_free(locked_pins);
 }
 
 static void sim_query_fac_pinlock_cb(const struct ofono_error *error,
@@ -3201,7 +3148,7 @@ static void sim_pin_query_cb(const struct ofono_error *error,
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(sim->atom);
 	struct cached_pin *cpins = pin_cache_lookup(sim->iccid);
-	const char *pin_name = sim_passwd_name(pin_type);
+	const char *pin_name;
 	char **locked_pins;
 	gboolean lock_changed;
 
@@ -3212,6 +3159,8 @@ static void sim_pin_query_cb(const struct ofono_error *error,
 		return;
 	}
 
+	pin_name = sim_passwd_name(pin_type);
+
 	if (sim->pin_type != pin_type) {
 		sim->pin_type = pin_type;
 
@@ -3234,7 +3183,7 @@ static void sim_pin_query_cb(const struct ofono_error *error,
 						"LockedPins", DBUS_TYPE_STRING,
 						&locked_pins);
 
-				g_strfreev(locked_pins);
+				l_strv_free(locked_pins);
 			}
 		}
 
@@ -3290,25 +3239,6 @@ void __ofono_sim_recheck_pin(struct ofono_sim *sim)
 	sim->driver->query_passwd_state(sim, sim_pin_query_cb, sim);
 }
 
-int ofono_sim_driver_register(const struct ofono_sim_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_sim_driver_unregister(const struct ofono_sim_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void emulator_remove_handler(struct ofono_atom *atom, void *data)
 {
 	struct ofono_emulator *em = __ofono_atom_get_data(atom);
@@ -3361,48 +3291,17 @@ static void sim_remove(struct ofono_atom *atom)
 	g_free(sim);
 }
 
-struct ofono_sim *ofono_sim_create(struct ofono_modem *modem,
-					unsigned int vendor,
-					const char *driver,
-					void *data)
-{
-	struct ofono_sim *sim;
-	GSList *l;
+OFONO_DEFINE_ATOM_CREATE(sim, OFONO_ATOM_TYPE_SIM, {
 	int i;
 
-	if (driver == NULL)
-		return NULL;
-
-	sim = g_try_new0(struct ofono_sim, 1);
-
-	if (sim == NULL)
-		return NULL;
-
-	sim->phase = OFONO_SIM_PHASE_UNKNOWN;
-	sim->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_SIM,
-						sim_remove, sim);
+	atom->phase = OFONO_SIM_PHASE_UNKNOWN;
 
 	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
-		sim->pin_retries[i] = -1;
+		atom->pin_retries[i] = -1;
 
-	sim->active_card_slot = 1;
-	sim->card_slot_count = 1;
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_sim_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(sim, vendor, data) < 0)
-			continue;
-
-		sim->driver = drv;
-		break;
-	}
-
-	return sim;
-}
+	atom->active_card_slot = 1;
+	atom->card_slot_count = 1;
+})
 
 static void emulator_cnum_cb(struct ofono_emulator *em,
 			struct ofono_emulator_request *req, void *userdata)
@@ -3802,7 +3701,8 @@ void __ofono_sim_remove_session_watch(struct ofono_sim_aid_session *session,
 {
 	__ofono_watchlist_remove_item(session->watches, id);
 
-	if (g_slist_length(session->watches->items) == 0) {
+	if (g_slist_length(session->watches->items) == 0 &&
+			session->state == SESSION_STATE_OPEN) {
 		/* last watcher, close session */
 		session->state = SESSION_STATE_CLOSING;
 		session->sim->driver->close_channel(session->sim,
