@@ -552,15 +552,13 @@ static DBusMessage *sms_set_property(DBusConnection *conn, DBusMessage *msg,
 
 		dbus_message_iter_get_basic(&var, &value);
 
-		if (strlen(value) == 0 || !valid_phone_number_format(value))
+		if (string_to_phone_number(value, &sca) < 0)
 			return __ofono_error_invalid_format(msg);
 
 		if (sms->driver->sca_set == NULL ||
 				sms->driver->sca_query == NULL)
 			return __ofono_error_not_implemented(msg);
 
-		string_to_phone_number(value, &sca);
-
 		sms->pending = dbus_message_ref(msg);
 
 		sms->driver->sca_set(sms, &sca, sca_set_callback, sms);
@@ -1779,7 +1777,7 @@ static void sms_remove(struct ofono_atom *atom)
 
 		storage_close(sms->imsi, SETTINGS_STORE, sms->settings, TRUE);
 
-		g_free(sms->imsi);
+		l_free(sms->imsi);
 		sms->imsi = NULL;
 		sms->settings = NULL;
 	}
@@ -1821,7 +1819,7 @@ static void sms_load_settings(struct ofono_sms *sms, const char *imsi)
 	if (sms->settings == NULL)
 		return;
 
-	sms->imsi = g_strdup(imsi);
+	sms->imsi = l_strdup(imsi);
 
 	error = NULL;
 	sms->ref = g_key_file_get_integer(sms->settings, SETTINGS_GROUP,
