@@ -34,12 +34,9 @@
 
 #include "common.h"
 #include "smsutil.h"
-#include "util.h"
 
 #define MAX_USSD_LENGTH 160
 
-static GSList *g_drivers = NULL;
-
 enum ussd_state {
 	USSD_STATE_IDLE = 0,
 	USSD_STATE_ACTIVE = 1,
@@ -93,7 +90,7 @@ static struct ssc_entry *ssc_entry_create(const char *sc, void *cb, void *data,
 	if (r == NULL)
 		return r;
 
-	r->service = g_strdup(sc);
+	r->service = l_strdup(sc);
 	r->cb = cb;
 	r->user = data;
 	r->destroy = destroy;
@@ -108,7 +105,7 @@ static void ssc_entry_destroy(gpointer pointer)
 	if (ca->destroy)
 		ca->destroy(ca->user);
 
-	g_free(ca->service);
+	l_free(ca->service);
 	g_free(ca);
 }
 
@@ -235,7 +232,7 @@ static gboolean recognized_control_string(struct ofono_ussd *ussd,
 						const char *ss_str,
 						DBusMessage *msg)
 {
-	char *str = g_strdup(ss_str);
+	char *str = l_strdup(ss_str);
 	char *sc, *sia, *sib, *sic, *sid, *dn;
 	int type;
 	gboolean ret = FALSE;
@@ -289,7 +286,7 @@ static gboolean recognized_control_string(struct ofono_ussd *ussd,
 	 */
 
 out:
-	g_free(str);
+	l_free(str);
 
 	return ret;
 }
@@ -784,25 +781,6 @@ static const GDBusSignalTable ussd_signals[] = {
 	{ }
 };
 
-int ofono_ussd_driver_register(const struct ofono_ussd_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_ussd_driver_unregister(const struct ofono_ussd_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void ussd_unregister(struct ofono_atom *atom)
 {
 	struct ofono_ussd *ussd = __ofono_atom_get_data(atom);
@@ -853,40 +831,7 @@ static void ussd_remove(struct ofono_atom *atom)
 	g_free(ussd);
 }
 
-struct ofono_ussd *ofono_ussd_create(struct ofono_modem *modem,
-					unsigned int vendor,
-					const char *driver,
-					void *data)
-{
-	struct ofono_ussd *ussd;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	ussd = g_try_new0(struct ofono_ussd, 1);
-
-	if (ussd == NULL)
-		return NULL;
-
-	ussd->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_USSD,
-						ussd_remove, ussd);
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_ussd_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(ussd, vendor, data) < 0)
-			continue;
-
-		ussd->driver = drv;
-		break;
-	}
-
-	return ussd;
-}
+OFONO_DEFINE_ATOM_CREATE(ussd, OFONO_ATOM_TYPE_USSD)
 
 void ofono_ussd_register(struct ofono_ussd *ussd)
 {
