@@ -411,14 +411,13 @@ static const struct cb_data testdata_set_passwd_invalid_1 = {
 };
 
 /* Declarations && Re-implementations of core functions. */
-void ril_call_barring_exit(void);
-void ril_call_barring_init(void);
-
 struct ofono_call_barring {
 	void *driver_data;
 	const struct cb_data *cbd;
 };
 
+extern struct ofono_driver_desc __start___call_barring[];
+
 struct ofono_call_barring *ofono_call_barring_create(struct ofono_modem *modem,
 							unsigned int vendor,
 							const char *driver,
@@ -428,20 +427,14 @@ struct ofono_call_barring *ofono_call_barring_create(struct ofono_modem *modem,
 	struct ofono_call_barring *cb = g_new0(struct ofono_call_barring, 1);
 	int retval;
 
+	cbdriver = __start___call_barring[0].driver;
+
 	retval = cbdriver->probe(cb, OFONO_RIL_VENDOR_AOSP, rsd->ril);
 	g_assert(retval == 0);
 
 	return cb;
 }
 
-int ofono_call_barring_driver_register(const struct ofono_call_barring_driver *d)
-{
-	if (cbdriver == NULL)
-		cbdriver = d;
-
-	return 0;
-}
-
 void ofono_call_barring_set_data(struct ofono_call_barring *cb, void *data)
 {
 	cb->driver_data = data;
@@ -456,10 +449,6 @@ void ofono_call_barring_register(struct ofono_call_barring *cb)
 {
 }
 
-void ofono_call_barring_driver_unregister(const struct ofono_call_barring_driver *d)
-{
-}
-
 static void server_connect_cb(gpointer data)
 {
 	struct rilmodem_cb_data *rsd = data;
@@ -494,8 +483,6 @@ static void test_call_barring_func(gconstpointer data)
 	const struct cb_data *sd = data;
 	struct rilmodem_cb_data *rsd;
 
-	ril_call_barring_init();
-
 	rsd = g_new0(struct rilmodem_cb_data, 1);
 
 	rsd->test_data = sd;
@@ -516,8 +503,6 @@ static void test_call_barring_func(gconstpointer data)
 	g_free(rsd);
 
 	rilmodem_test_server_close(rsd->serverd);
-
-	ril_call_barring_exit();
 }
 
 #endif
